﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using nEwGEN.Odf;
using System.CodeDom.Compiler;
using Microsoft.CSharp;
using System.Reflection;
using System.Text.RegularExpressions;

namespace FleetOpsEditor
{
    public partial class FOEMainForm : Form
    {
        List<CustomClass> openODFs = new List<CustomClass>();
        List<OdfDocument> odfDocsOpen = new List<OdfDocument>();
        List<Regex> weaponRegexes = new List<Regex>();
        public FOEMainForm()
        {
            InitializeComponent();
            weaponRegexes.Add(new Regex("weapon[0-9]"));
            weaponRegexes.Add(new Regex("weaponHardpoints[0-9]"));
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openODFs.Count != 0)
            {
                openODFs = new List<CustomClass>();
                odfDocsOpen = new List<OdfDocument>();
                selectedFileBox.Items.Clear();
            }
            DialogResult result = odfFileOpener.ShowDialog();
            if (result == DialogResult.OK)
            {
                OdfDocument doc;
                CustomClass baseODF = LoadODF(odfFileOpener.FileName, out doc);
                openODFs.Add(baseODF);
                odfDocsOpen.Add(doc);
                /*bool continueLoading = doc.AttributeExists("nextrankClass");
                //while (continueLoading != false)
                {
                    object rankedODF = LoadODF(Path.GetDirectoryName(odfFileOpener.FileName) + "\\" +
                                                doc.GetAttribute("nextrankClass", OdfAttributeType.String) + ".odf", 
                                                out doc);
                    openODFs.Add(rankedODF);
                    odfDocsOpen.Add(doc);
                    continueLoading = doc.AttributeExists("nextrankClass");
                }*/
                mainEditor.PropertySort = PropertySort.CategorizedAlphabetical;
                mainEditor.SelectedObject = baseODF;
                for (int i = 0; i < openODFs.Count; i++)
                {
                    selectedFileBox.Items.Add("Rank selected: " + (i + 1));
                }
                selectedFileBox.SelectedIndex = 0;
            }
        }

        public CustomClass LoadODF(string pathAndName, out OdfDocument outDoc)
        {
            OdfDocument document = new OdfDocument(pathAndName);
            outDoc = document;
            /*
            string code = @"using System;
                            using System.ComponentModel;
                                class RuntimeOdfObject {";
            code += "\n";
            foreach (string attribute in document.GetAttributeList())
            {
                OdfAttributeType actualType = OdfAttributeType.Int;
                string result2 = "";
                foreach (string name in Enum.GetNames(typeof(OdfAttributeType)))
                {
                    result2 = document.GetAttribute(attribute, (OdfAttributeType)Enum.Parse(typeof(OdfAttributeType), name));
                    if (result2 != "")
                    {
                        actualType = (OdfAttributeType)Enum.Parse(typeof(OdfAttributeType), name);
                        break;
                    }
                }
                string attributeType = ConvertOdfTypeToCode(actualType);
                //there are more but i dont have the time to find them right now
                //aside from the fact that we need to replace some of the conditions here...
                if (attribute != "possibleCraftNames" && !new Regex("weaponsHardpoints[0-9]").IsMatch(attribute))
                {
                    code += "private " + attributeType + "_" + attribute + " = " + EscapeOdfString(actualType, document.GetAttribute(attribute, actualType)) + ";\n";
                    code += "[CategoryAttribute(\"" + GetCategory(attribute) + "\")]\n";
                    code += "public " + attributeType + attribute + " { get { return _" + attribute + "; } set { _" + attribute + " = value; } }\n";
                }
                else
                {
                    string[] temp = document.GetAttributeArray(attribute, OdfAttributeArrayType.StringMultiLine);
                    if (temp.Length == 0)
                    {
                       temp = document.GetAttributeArray(attribute, OdfAttributeArrayType.StringMultiLine);
                    }
                    code += "private string[] _" + attribute + " = {";
                    foreach (string str in temp)
                    {
                        code += "\"" + str + "\", ";
                    }
                    code += "};\n";

                }
            }
            code += "}";

            ICodeCompiler compiler = new CSharpCodeProvider().CreateCompiler();
            CompilerParameters cparams = new CompilerParameters();
            cparams.ReferencedAssemblies.Add("System.dll");
            //cparams.GenerateInMemory = false;
            cparams.GenerateExecutable = false;
            CompilerResults results = compiler.CompileAssemblyFromSource(cparams, code);
            if (results.Errors.HasErrors)
                throw new NotImplementedException();
            Assembly assembly = results.CompiledAssembly;
            object odf = assembly.CreateInstance("RuntimeOdfObject");
             */
            CustomClass odf = new CustomClass();
            foreach (string attribute in document.GetAttributeList())
            {
                OdfAttributeType actualType = OdfAttributeType.Int;
                string result2 = "";
                foreach (string name in Enum.GetNames(typeof(OdfAttributeType)))
                {
                    result2 = document.GetAttribute(attribute, (OdfAttributeType)Enum.Parse(typeof(OdfAttributeType), name));
                    if (result2 != "")
                    {
                        actualType = (OdfAttributeType)Enum.Parse(typeof(OdfAttributeType), name);
                        break;
                    }
                }
                string attributeType = ConvertOdfTypeToCode(actualType);
                bool array = false;
                foreach (OdfAttributeArrayType type in Enum.GetValues(typeof(OdfAttributeArrayType)))
                {
                    string [] temp1 = new string[1];
                    try
                    {
                        temp1 = document.GetAttributeArray(attribute, type);
                    }
                    catch (Exception e)
                    {
                        //if we're here, then we definitely want to process this as an array...
                        temp1 = new string[] { "doesn't matter", "what's in these" };
                    }
                    if (temp1.Length > 1)
                    {
                        array = true;
                        break;
                    }
                }
                if (array == false)
                {

                    CustomProperty curProp = new CustomProperty("temp", 0, true, false);
                    if (actualType == OdfAttributeType.Boolean)
                    {
                        curProp = new CustomProperty(attribute, Convert.ToBoolean(document.GetAttribute(attribute, actualType), System.Globalization.CultureInfo.InstalledUICulture), false, true);
                    }
                    else if (actualType == OdfAttributeType.Float || actualType == OdfAttributeType.FloatF)
                    {
                        curProp = new CustomProperty(attribute, (float)Convert.ToDouble(document.GetAttribute(attribute, actualType), System.Globalization.CultureInfo.InvariantCulture), false, true);
                    }
                    else if (actualType == OdfAttributeType.Int)
                    {
                        curProp = new CustomProperty(attribute, Convert.ToInt32(document.GetAttribute(attribute, actualType), System.Globalization.CultureInfo.InstalledUICulture), false, true);
                    }
                    else if (actualType == OdfAttributeType.String)
                    {
                        curProp = new CustomProperty(attribute, document.GetAttribute(attribute, actualType), false, true);
                    }
                    odf.Add(curProp);
                }
                else
                {
                    CustomProperty curProp = new CustomProperty("temp", 0, true, false);
                    string[] temp = new string[1];
                    foreach (OdfAttributeArrayType type in Enum.GetValues(typeof(OdfAttributeArrayType)))
                    {
                        try
                        {
                            temp = document.GetAttributeArray(attribute, type);
                        }
                        catch (Exception e)
                        {
                            temp = new string[] { "Attribute array is too long", "Please shorten this attribute's array in an external editor" };
                        }
                        if (temp.Length > 1)
                        {
                            break;
                        }
                    }
                    if (temp.Length > 0)
                    {
                        curProp = new CustomProperty(attribute, temp, false, true);
                    }
                    odf.Add(curProp);
                }
            }
            return odf;
        }

        /*public string GetCategory(string attribute)
        {
            foreach (Regex r in weaponRegexes)
            {
                if (r.IsMatch(attribute))
                {
                    return "Weapons";
                }
            }
            return "Uncategorized";
        }*/

        /*private string EscapeOdfString(OdfAttributeType type, string data)
        {
            if (type == OdfAttributeType.String)
                return "\"" + data + "\"";
            else
                return data;
        }*/

        private string ConvertOdfTypeToCode(OdfAttributeType actualType)
        {
            switch (actualType)
            {
                case OdfAttributeType.Boolean:
                    return "bool ";
                case OdfAttributeType.Float:
                    return "float ";
                case OdfAttributeType.FloatF:
                    return "float ";
                case OdfAttributeType.Int:
                    return "int ";
                case OdfAttributeType.String:
                    return "string ";
            }

            return "string ";
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (odfFileSaver.ShowDialog() == DialogResult.OK)
            {
                string nextRankName = "";
                for (int i = 0; i < odfDocsOpen.Count; i++)
                {
                    foreach (CustomProperty property in openODFs[i])
                    {
                        if (property.Value.GetType().IsArray)
                        {
                            if (property.Value.GetType() != typeof(string[]))
                            {
                                string[] temp = new string[(property.Value as object[]).Length];
                                for (int j = 0; j < (property.Value as object[]).Length; j++)
                                {
                                    temp[j] = Convert.ToString((property.Value as object[])[j]);
                                }
                                odfDocsOpen[i].SetAttributeArray(property.Name, temp, FrameworkTypeToOdfArrayType(property.Value.GetType()));
                            }
                            else
                            {
                                odfDocsOpen[i].SetAttributeArray(property.Name, (property.Value as string[]), FrameworkTypeToOdfArrayType(property.Value.GetType()));
                            }
                        }
                        else
                        {
                            odfDocsOpen[i].SetAttribute(property.Name, Convert.ToString(property.Value), FrameworkTypeToOdfType(property.Value.GetType()));
                        }
                    }
                    if (i == 0)
                    {
                        if (odfDocsOpen[i].AttributeExists("nextrankClass"))
                        {
                            nextRankName = odfDocsOpen[i].GetAttribute("nextrankClass", OdfAttributeType.String);
                        }
                        odfDocsOpen[i].Save(odfFileSaver.FileName);
                    }
                    else
                    {
                        string path = Path.GetDirectoryName(odfFileSaver.FileName) + "\\" + nextRankName + ".odf";
                        if (odfDocsOpen[i].AttributeExists("nextrankClass"))
                        {
                            nextRankName = odfDocsOpen[i].GetAttribute("nextrankClass", OdfAttributeType.String);
                        }
                        odfDocsOpen[i].Save(path);
                    }
                }
            }
        }

        private OdfAttributeType FrameworkTypeToOdfType(Type t)
        {
            if (t == typeof(string))
            {
                    return OdfAttributeType.String;
            }
            else if (t == typeof(float))
            {
                    return OdfAttributeType.Float;
            }
            else if (t == typeof(bool))
            {
                    return OdfAttributeType.Boolean;
            }
            else if (t == typeof(int))
            {
                return OdfAttributeType.Int;
            }
            return OdfAttributeType.String;
        }

        private OdfAttributeArrayType FrameworkTypeToOdfArrayType(Type t)
        {
            if (t == typeof(string[]))
            {
                return OdfAttributeArrayType.String;
            }
            else if (t == typeof(float[]))
            {
                return OdfAttributeArrayType.Float;
            }
            else if (t == typeof(int[]))
            {
                return OdfAttributeArrayType.Int;
            }
            return OdfAttributeArrayType.FloatF;
        }

        void rankBox_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            mainEditor.SelectedObject = openODFs[selectedFileBox.SelectedIndex];
        }
    }
}
